@@ -1,15 +1,15 @@
 use std::borrow::Cow;
 use std::fmt::{Display, Formatter};
 use std::ops::Deref;
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 use std::str::FromStr;
 
 use itertools::Either;
 use once_cell::sync::Lazy;
-use serde::{Deserialize, Serialize};
-use url::Url;
+use thiserror::Error;
+use url::{ParseError, Url};
 
-use pep508_rs::{expand_env_vars, split_scheme, strip_host, Scheme, VerbatimUrl};
+use pep508_rs::{expand_env_vars, split_scheme, strip_host, Scheme, VerbatimUrl, VerbatimUrlError};
 use uv_fs::normalize_url_path;
 
 use crate::Verbatim;
@@ -19,7 +19,7 @@ static PYPI_URL: Lazy<Url> = Lazy::new(|| Url::parse("https://pypi.org/simple").
 static DEFAULT_INDEX_URL: Lazy<IndexUrl> =
     Lazy::new(|| IndexUrl::Pypi(VerbatimUrl::from_url(PYPI_URL.clone())));
 
-/// The url of an index, newtype'd to avoid mixing it with file urls.
+/// The URL of an index to use for fetching packages (e.g., PyPI).
 #[derive(Debug, Clone, Hash, Eq, PartialEq)]
 pub enum IndexUrl {
     Pypi(VerbatimUrl),
@@ -36,6 +36,11 @@ impl schemars::JsonSchema for IndexUrl {
     fn json_schema(_gen: &mut schemars::gen::SchemaGenerator) -> schemars::schema::Schema {
         schemars::schema::SchemaObject {
             instance_type: Some(schemars::schema::InstanceType::String.into()),
+            format: Some("uri".to_owned()),
+            metadata: Some(Box::new(schemars::schema::Metadata {
+                description: Some("The URL of an index to use for fetching packages (e.g., `https://pypi.org/simple`).".to_string()),
+              ..schemars::schema::Metadata::default()
+            })),
             ..schemars::schema::SchemaObject::default()
         }
         .into()
@@ -86,17 +91,27 @@ impl Verbatim for IndexUrl {
     }
 }
 
+/// An error that can occur when parsing an [`IndexUrl`].
+#[derive(Error, Debug)]
+pub enum IndexUrlError {
+    #[error(transparent)]
+    Io(#[from] std::io::Error),
+    #[error(transparent)]
+    Url(#[from] ParseError),
+    #[error(transparent)]
+    VerbatimUrl(#[from] VerbatimUrlError),
+}
+
 impl FromStr for IndexUrl {
-    type Err = url::ParseError;
+    type Err = IndexUrlError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        let url = Url::parse(s)?;
-        let url = VerbatimUrl::from_url(url).with_given(s.to_owned());
-        if *url.raw() == *PYPI_URL {
-            Ok(Self::Pypi(url))
+        let url = if let Ok(path) = Path::new(s).canonicalize() {
+            VerbatimUrl::from_path(path)?
         } else {
-            Ok(Self::Url(url))
-        }
+            VerbatimUrl::parse_url(s)?
+        };
+        Ok(Self::from(url.with_given(s)))
     }
 }
 
@@ -105,7 +120,7 @@ impl serde::ser::Serialize for IndexUrl {
     where
         S: serde::ser::Serializer,
     {
-        self.verbatim().serialize(serializer)
+        self.to_string().serialize(serializer)
     }
 }
 
@@ -121,7 +136,9 @@ impl<'de> serde::de::Deserialize<'de> for IndexUrl {
 
 impl From<VerbatimUrl> for IndexUrl {
     fn from(url: VerbatimUrl) -> Self {
-        if *url.raw() == *PYPI_URL {
+        if url.scheme() == "file" {
+            Self::Path(url)
+        } else if *url.raw() == *PYPI_URL {
             Self::Pypi(url)
         } else {
             Self::Url(url)
@@ -154,7 +171,7 @@ impl Deref for IndexUrl {
 /// A directory with distributions or a URL to an HTML file with a flat listing of distributions.
 ///
 /// Also known as `--find-links`.
-#[derive(Debug, Clone, Hash, Eq, PartialEq, Serialize, Deserialize)]
+#[derive(Debug, Clone, Hash, Eq, PartialEq)]
 pub enum FlatIndexLocation {
     Path(PathBuf),
     Url(Url),
@@ -169,12 +186,36 @@ impl schemars::JsonSchema for FlatIndexLocation {
     fn json_schema(_gen: &mut schemars::gen::SchemaGenerator) -> schemars::schema::Schema {
         schemars::schema::SchemaObject {
             instance_type: Some(schemars::schema::InstanceType::String.into()),
+            format: Some("uri".to_owned()),
+            metadata: Some(Box::new(schemars::schema::Metadata {
+                description: Some("The path to a directory of distributions, or a URL to an HTML file with a flat listing of distributions.".to_string()),
+              ..schemars::schema::Metadata::default()
+            })),
             ..schemars::schema::SchemaObject::default()
         }
         .into()
     }
 }
 
+impl serde::ser::Serialize for FlatIndexLocation {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::ser::Serializer,
+    {
+        self.to_string().serialize(serializer)
+    }
+}
+
+impl<'de> serde::de::Deserialize<'de> for FlatIndexLocation {
+    fn deserialize<D>(deserializer: D) -> Result<FlatIndexLocation, D::Error>
+    where
+        D: serde::de::Deserializer<'de>,
+    {
+        let s = String::deserialize(deserializer)?;
+        FlatIndexLocation::from_str(&s).map_err(serde::de::Error::custom)
+    }
+}
+
 impl FromStr for FlatIndexLocation {
     type Err = url::ParseError;
 
