@@ -11,20 +11,21 @@ use clap::{CommandFactory, Parser};
 use owo_colors::OwoColorize;
 use tracing::{debug, instrument};
 
-use cli::{ToolCommand, ToolNamespace, ToolchainCommand, ToolchainNamespace};
+use settings::PipTreeSettings;
 use uv_cache::Cache;
+use uv_cli::{
+    compat::CompatArgs, CacheCommand, CacheNamespace, Cli, Commands, PipCommand, PipNamespace,
+    ProjectCommand,
+};
+#[cfg(feature = "self-update")]
+use uv_cli::{SelfCommand, SelfNamespace};
+use uv_cli::{ToolCommand, ToolNamespace, ToolchainCommand, ToolchainNamespace};
 use uv_configuration::Concurrency;
 use uv_distribution::Workspace;
 use uv_requirements::RequirementsSource;
 use uv_settings::Combine;
 
-use crate::cli::{
-    CacheCommand, CacheNamespace, Cli, Commands, PipCommand, PipNamespace, ProjectCommand,
-};
-#[cfg(feature = "self-update")]
-use crate::cli::{SelfCommand, SelfNamespace};
 use crate::commands::ExitStatus;
-use crate::compat::CompatArgs;
 use crate::settings::{
     CacheSettings, GlobalSettings, PipCheckSettings, PipCompileSettings, PipFreezeSettings,
     PipInstallSettings, PipListSettings, PipShowSettings, PipSyncSettings, PipUninstallSettings,
@@ -46,9 +47,7 @@ static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
 #[global_allocator]
 static GLOBAL: tikv_jemallocator::Jemalloc = tikv_jemallocator::Jemalloc;
 
-mod cli;
 mod commands;
-mod compat;
 mod logging;
 mod printer;
 mod settings;
@@ -105,6 +104,12 @@ async fn run() -> Result<ExitStatus> {
                             ContextValue::String("uv pip show".to_string()),
                         );
                     }
+                    "tree" => {
+                        err.insert(
+                            ContextKind::SuggestedSubcommand,
+                            ContextValue::String("uv pip tree".to_string()),
+                        );
+                    }
                     _ => {}
                 }
             }
@@ -140,7 +145,7 @@ async fn run() -> Result<ExitStatus> {
     };
 
     // Resolve the global settings.
-    let globals = GlobalSettings::resolve(&cli.global_args, filesystem.as_ref());
+    let globals = GlobalSettings::resolve(&cli.command, &cli.global_args, filesystem.as_ref());
 
     // Resolve the cache settings.
     let cache_settings = CacheSettings::resolve(cli.cache_args, filesystem.as_ref());
@@ -258,11 +263,13 @@ async fn run() -> Result<ExitStatus> {
                 args.settings.generate_hashes,
                 args.settings.no_emit_package,
                 args.settings.no_strip_extras,
+                args.settings.no_strip_markers,
                 !args.settings.no_annotate,
                 !args.settings.no_header,
                 args.settings.custom_compile_command,
                 args.settings.emit_index_url,
                 args.settings.emit_find_links,
+                args.settings.emit_build_options,
                 args.settings.emit_marker_expression,
                 args.settings.emit_index_annotation,
                 args.settings.index_locations,
@@ -275,11 +282,13 @@ async fn run() -> Result<ExitStatus> {
                 args.settings.build_options,
                 args.settings.python_version,
                 args.settings.python_platform,
+                args.settings.universal,
                 args.settings.exclude_newer,
                 args.settings.annotation_style,
                 args.settings.link_mode,
                 args.settings.python,
                 args.settings.system,
+                globals.toolchain_preference,
                 args.settings.concurrency,
                 globals.native_tls,
                 globals.quiet,
@@ -529,6 +538,25 @@ async fn run() -> Result<ExitStatus> {
                 printer,
             )
         }
+        Commands::Pip(PipNamespace {
+            command: PipCommand::Tree(args),
+        }) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = PipTreeSettings::resolve(args, filesystem);
+
+            // Initialize the cache.
+            let cache = cache.init()?;
+
+            commands::pip_tree(
+                args.no_dedupe,
+                args.shared.strict,
+                args.shared.python.as_deref(),
+                args.shared.system,
+                globals.preview,
+                &cache,
+                printer,
+            )
+        }
         Commands::Pip(PipNamespace {
             command: PipCommand::Check(args),
         }) => {
@@ -585,6 +613,7 @@ async fn run() -> Result<ExitStatus> {
             commands::venv(
                 &args.name,
                 args.settings.python.as_deref(),
+                globals.toolchain_preference,
                 args.settings.link_mode,
                 &args.settings.index_locations,
                 args.settings.index_strategy,
@@ -619,14 +648,14 @@ async fn run() -> Result<ExitStatus> {
             commands::run(
                 args.extras,
                 args.dev,
-                args.target,
-                args.args,
+                args.command,
                 requirements,
                 args.python,
                 args.package,
                 args.settings,
                 globals.isolated,
                 globals.preview,
+                globals.toolchain_preference,
                 globals.connectivity,
                 Concurrency::default(),
                 globals.native_tls,
@@ -648,6 +677,7 @@ async fn run() -> Result<ExitStatus> {
                 args.dev,
                 args.modifications,
                 args.python,
+                globals.toolchain_preference,
                 args.settings,
                 globals.preview,
                 globals.connectivity,
@@ -670,6 +700,7 @@ async fn run() -> Result<ExitStatus> {
                 args.python,
                 args.settings,
                 globals.preview,
+                globals.toolchain_preference,
                 globals.connectivity,
                 Concurrency::default(),
                 globals.native_tls,
@@ -688,10 +719,16 @@ async fn run() -> Result<ExitStatus> {
 
             commands::add(
                 args.requirements,
-                args.workspace,
                 args.dev,
+                args.editable,
+                args.raw_sources,
+                args.rev,
+                args.tag,
+                args.branch,
+                args.package,
                 args.python,
                 args.settings,
+                globals.toolchain_preference,
                 globals.preview,
                 globals.connectivity,
                 Concurrency::default(),
@@ -712,7 +749,9 @@ async fn run() -> Result<ExitStatus> {
             commands::remove(
                 args.requirements,
                 args.dev,
+                args.package,
                 args.python,
+                globals.toolchain_preference,
                 globals.preview,
                 globals.connectivity,
                 Concurrency::default(),
@@ -745,14 +784,41 @@ async fn run() -> Result<ExitStatus> {
             let cache = cache.init()?.with_refresh(args.refresh);
 
             commands::run_tool(
-                args.target,
-                args.args,
+                args.command,
                 args.python,
                 args.from,
                 args.with,
                 args.settings,
                 globals.isolated,
                 globals.preview,
+                globals.toolchain_preference,
+                globals.connectivity,
+                Concurrency::default(),
+                globals.native_tls,
+                &cache,
+                printer,
+            )
+            .await
+        }
+        Commands::Tool(ToolNamespace {
+            command: ToolCommand::Install(args),
+        }) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::ToolInstallSettings::resolve(args, filesystem);
+            show_settings!(args);
+
+            // Initialize the cache.
+            let cache = cache.init()?.with_refresh(args.refresh);
+
+            commands::tool_install(
+                args.name,
+                args.python,
+                args.from,
+                args.with,
+                args.force,
+                args.settings,
+                globals.preview,
+                globals.toolchain_preference,
                 globals.connectivity,
                 Concurrency::default(),
                 globals.native_tls,
@@ -775,6 +841,7 @@ async fn run() -> Result<ExitStatus> {
                 args.kinds,
                 args.all_versions,
                 args.all_platforms,
+                globals.toolchain_preference,
                 globals.preview,
                 &cache,
                 printer,
@@ -811,7 +878,14 @@ async fn run() -> Result<ExitStatus> {
             // Initialize the cache.
             let cache = cache.init()?;
 
-            commands::toolchain_find(args.request, globals.preview, &cache, printer).await
+            commands::toolchain_find(
+                args.request,
+                globals.toolchain_preference,
+                globals.preview,
+                &cache,
+                printer,
+            )
+            .await
         }
     }
 }
