@@ -32,10 +32,10 @@ fn add_registry() -> Result<()> {
     ----- stderr -----
     warning: `uv add` is experimental and may change without warning.
     Resolved 4 packages in [TIME]
-    Downloaded 4 packages in [TIME]
+    Prepared 4 packages in [TIME]
     Installed 4 packages in [TIME]
      + anyio==3.7.0
-     + idna==3.7
+     + idna==3.6
      + project==0.1.0 (from file://[TEMP_DIR]/)
      + sniffio==1.3.1
     "###);
@@ -78,31 +78,24 @@ fn add_registry() -> Result<()> {
 
         [[distribution.dependencies]]
         name = "idna"
-        version = "3.7"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution.dependencies]]
         name = "sniffio"
-        version = "1.3.1"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution]]
         name = "idna"
-        version = "3.7"
+        version = "3.6"
         source = "registry+https://pypi.org/simple"
-        sdist = { url = "https://files.pythonhosted.org/packages/21/ed/f86a79a07470cb07819390452f178b3bef1d375f2ec021ecfc709fc7cf07/idna-3.7.tar.gz", hash = "sha256:028ff3aadf0609c1fd278d8ea3089299412a7a8b9bd005dd08b9f8285bcb5cfc", size = 189575 }
-        wheels = [{ url = "https://files.pythonhosted.org/packages/e5/3e/741d8c82801c347547f8a2a06aa57dbb1992be9e948df2ea0eda2c8b79e8/idna-3.7-py3-none-any.whl", hash = "sha256:82fee1fc78add43492d3a1898bfa6d8a904cc97d8427f683ed8e798d07761aa0", size = 66836 }]
+        sdist = { url = "https://files.pythonhosted.org/packages/bf/3f/ea4b9117521a1e9c50344b909be7886dd00a519552724809bb1f486986c2/idna-3.6.tar.gz", hash = "sha256:9ecdbbd083b06798ae1e86adcbfe8ab1479cf864e4ee30fe4e46a003d12491ca", size = 175426 }
+        wheels = [{ url = "https://files.pythonhosted.org/packages/c2/e7/a82b05cf63a603df6e68d59ae6a68bf5064484a0718ea5033660af4b54a9/idna-3.6-py3-none-any.whl", hash = "sha256:c05567e9c24a6b9faaa835c4821bad0590fbb9d5779e7caa6e1cc4978e7eb24f", size = 61567 }]
 
         [[distribution]]
         name = "project"
         version = "0.1.0"
         source = "editable+."
-        sdist = { path = "." }
 
         [[distribution.dependencies]]
         name = "anyio"
-        version = "3.7.0"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution]]
         name = "sniffio"
@@ -159,7 +152,7 @@ fn add_git() -> Result<()> {
 
     ----- stderr -----
     warning: `uv sync` is experimental and may change without warning.
-    Downloaded 4 packages in [TIME]
+    Prepared 4 packages in [TIME]
     Installed 4 packages in [TIME]
      + anyio==3.7.0
      + idna==3.6
@@ -167,15 +160,171 @@ fn add_git() -> Result<()> {
      + sniffio==1.3.1
     "###);
 
-    uv_snapshot!(context.filters(), context.add(&["uv-public-pypackage @ git+https://github.com/astral-test/uv-public-pypackage@0.0.1"]), @r###"
+    // Adding with an ambiguous Git reference will fail.
+    uv_snapshot!(context.filters(), context.add(&["uv-public-pypackage @ git+https://github.com/astral-test/uv-public-pypackage@0.0.1"]).arg("--preview"), @r###"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    error: Cannot resolve Git reference `0.0.1` for requirement `uv-public-pypackage`. Specify the reference with one of `--tag`, `--branch`, or `--rev`, or use the `--raw-sources` flag.
+    "###);
+
+    uv_snapshot!(context.filters(), context.add(&["uv-public-pypackage @ git+https://github.com/astral-test/uv-public-pypackage"]).arg("--tag=0.0.1").arg("--preview"), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
-    warning: `uv add` is experimental and may change without warning.
     Resolved 5 packages in [TIME]
-    Downloaded 2 packages in [TIME]
+    Prepared 2 packages in [TIME]
+    Uninstalled 1 package in [TIME]
+    Installed 2 packages in [TIME]
+     - project==0.1.0 (from file://[TEMP_DIR]/)
+     + project==0.1.0 (from file://[TEMP_DIR]/)
+     + uv-public-pypackage==0.1.0 (from git+https://github.com/astral-test/uv-public-pypackage@0dacfd662c64cb4ceb16e6cf65a157a8b715b979?tag=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979)
+    "###);
+
+    let pyproject_toml = fs_err::read_to_string(context.temp_dir.join("pyproject.toml"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            pyproject_toml, @r###"
+        [project]
+        name = "project"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = [
+            "anyio==3.7.0",
+            "uv-public-pypackage",
+        ]
+
+        [tool.uv.sources]
+        uv-public-pypackage = { git = "https://github.com/astral-test/uv-public-pypackage", tag = "0.0.1" }
+        "###
+        );
+    });
+
+    let lock = fs_err::read_to_string(context.temp_dir.join("uv.lock"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            lock, @r###"
+        version = 1
+        requires-python = ">=3.12"
+
+        [[distribution]]
+        name = "anyio"
+        version = "3.7.0"
+        source = "registry+https://pypi.org/simple"
+        sdist = { url = "https://files.pythonhosted.org/packages/c6/b3/fefbf7e78ab3b805dec67d698dc18dd505af7a18a8dd08868c9b4fa736b5/anyio-3.7.0.tar.gz", hash = "sha256:275d9973793619a5374e1c89a4f4ad3f4b0a5510a2b5b939444bee8f4c4d37ce", size = 142737 }
+        wheels = [{ url = "https://files.pythonhosted.org/packages/68/fe/7ce1926952c8a403b35029e194555558514b365ad77d75125f521a2bec62/anyio-3.7.0-py3-none-any.whl", hash = "sha256:eddca883c4175f14df8aedce21054bfca3adb70ffe76a9f607aef9d7fa2ea7f0", size = 80873 }]
+
+        [[distribution.dependencies]]
+        name = "idna"
+
+        [[distribution.dependencies]]
+        name = "sniffio"
+
+        [[distribution]]
+        name = "idna"
+        version = "3.6"
+        source = "registry+https://pypi.org/simple"
+        sdist = { url = "https://files.pythonhosted.org/packages/bf/3f/ea4b9117521a1e9c50344b909be7886dd00a519552724809bb1f486986c2/idna-3.6.tar.gz", hash = "sha256:9ecdbbd083b06798ae1e86adcbfe8ab1479cf864e4ee30fe4e46a003d12491ca", size = 175426 }
+        wheels = [{ url = "https://files.pythonhosted.org/packages/c2/e7/a82b05cf63a603df6e68d59ae6a68bf5064484a0718ea5033660af4b54a9/idna-3.6-py3-none-any.whl", hash = "sha256:c05567e9c24a6b9faaa835c4821bad0590fbb9d5779e7caa6e1cc4978e7eb24f", size = 61567 }]
+
+        [[distribution]]
+        name = "project"
+        version = "0.1.0"
+        source = "editable+."
+
+        [[distribution.dependencies]]
+        name = "anyio"
+
+        [[distribution.dependencies]]
+        name = "uv-public-pypackage"
+
+        [[distribution]]
+        name = "sniffio"
+        version = "1.3.1"
+        source = "registry+https://pypi.org/simple"
+        sdist = { url = "https://files.pythonhosted.org/packages/a2/87/a6771e1546d97e7e041b6ae58d80074f81b7d5121207425c964ddf5cfdbd/sniffio-1.3.1.tar.gz", hash = "sha256:f4324edc670a0f49750a81b895f35c3adb843cca46f0530f79fc1babb23789dc", size = 20372 }
+        wheels = [{ url = "https://files.pythonhosted.org/packages/e9/44/75a9c9421471a6c4805dbf2356f7c181a29c1879239abab1ea2cc8f38b40/sniffio-1.3.1-py3-none-any.whl", hash = "sha256:2f6da418d1f1e0fddd844478f41680e794e6051915791a034ff65e5f100525a2", size = 10235 }]
+
+        [[distribution]]
+        name = "uv-public-pypackage"
+        version = "0.1.0"
+        source = "git+https://github.com/astral-test/uv-public-pypackage?tag=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979"
+        "###
+        );
+    });
+
+    // Install from the lockfile.
+    uv_snapshot!(context.filters(), context.sync(), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv sync` is experimental and may change without warning.
+    Audited 5 packages in [TIME]
+    "###);
+
+    Ok(())
+}
+
+/// Add a Git requirement using the `--raw-sources` API.
+#[test]
+fn add_git_raw() -> Result<()> {
+    let context = TestContext::new("3.12");
+
+    let pyproject_toml = context.temp_dir.child("pyproject.toml");
+    pyproject_toml.write_str(indoc! {r#"
+        [project]
+        name = "project"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = ["anyio==3.7.0"]
+    "#})?;
+
+    uv_snapshot!(context.filters(), context.lock(), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv lock` is experimental and may change without warning.
+    Resolved 4 packages in [TIME]
+    "###);
+
+    uv_snapshot!(context.filters(), context.sync(), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv sync` is experimental and may change without warning.
+    Prepared 4 packages in [TIME]
+    Installed 4 packages in [TIME]
+     + anyio==3.7.0
+     + idna==3.6
+     + project==0.1.0 (from file://[TEMP_DIR]/)
+     + sniffio==1.3.1
+    "###);
+
+    // Use an ambiguous tag reference, which would otherwise not resolve.
+    uv_snapshot!(context.filters(), context.add(&["uv-public-pypackage @ git+https://github.com/astral-test/uv-public-pypackage@0.0.1"]).arg("--raw-sources").arg("--preview"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Resolved 5 packages in [TIME]
+    Prepared 2 packages in [TIME]
     Uninstalled 1 package in [TIME]
     Installed 2 packages in [TIME]
      - project==0.1.0 (from file://[TEMP_DIR]/)
@@ -221,13 +370,9 @@ fn add_git() -> Result<()> {
 
         [[distribution.dependencies]]
         name = "idna"
-        version = "3.6"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution.dependencies]]
         name = "sniffio"
-        version = "1.3.1"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution]]
         name = "idna"
@@ -240,17 +385,12 @@ fn add_git() -> Result<()> {
         name = "project"
         version = "0.1.0"
         source = "editable+."
-        sdist = { path = "." }
 
         [[distribution.dependencies]]
         name = "anyio"
-        version = "3.7.0"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution.dependencies]]
         name = "uv-public-pypackage"
-        version = "0.1.0"
-        source = "git+https://github.com/astral-test/uv-public-pypackage?rev=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979"
 
         [[distribution]]
         name = "sniffio"
@@ -263,7 +403,6 @@ fn add_git() -> Result<()> {
         name = "uv-public-pypackage"
         version = "0.1.0"
         source = "git+https://github.com/astral-test/uv-public-pypackage?rev=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979"
-        sdist = { url = "https://github.com/astral-test/uv-public-pypackage?rev=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979" }
         "###
         );
     });
@@ -297,18 +436,17 @@ fn add_unnamed() -> Result<()> {
         dependencies = []
     "#})?;
 
-    uv_snapshot!(context.filters(), context.add(&["git+https://github.com/astral-test/uv-public-pypackage@0.0.1"]), @r###"
+    uv_snapshot!(context.filters(), context.add(&["git+https://github.com/astral-test/uv-public-pypackage"]).arg("--tag=0.0.1").arg("--preview"), @r###"
     success: true
     exit_code: 0
     ----- stdout -----
 
     ----- stderr -----
-    warning: `uv add` is experimental and may change without warning.
     Resolved 2 packages in [TIME]
-    Downloaded 2 packages in [TIME]
+    Prepared 2 packages in [TIME]
     Installed 2 packages in [TIME]
      + project==0.1.0 (from file://[TEMP_DIR]/)
-     + uv-public-pypackage==0.1.0 (from git+https://github.com/astral-test/uv-public-pypackage@0dacfd662c64cb4ceb16e6cf65a157a8b715b979?rev=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979)
+     + uv-public-pypackage==0.1.0 (from git+https://github.com/astral-test/uv-public-pypackage@0dacfd662c64cb4ceb16e6cf65a157a8b715b979?tag=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979)
     "###);
 
     let pyproject_toml = fs_err::read_to_string(context.temp_dir.join("pyproject.toml"))?;
@@ -324,8 +462,11 @@ fn add_unnamed() -> Result<()> {
         # ...
         requires-python = ">=3.12"
         dependencies = [
-            "uv-public-pypackage @ git+https://github.com/astral-test/uv-public-pypackage@0.0.1",
+            "uv-public-pypackage",
         ]
+
+        [tool.uv.sources]
+        uv-public-pypackage = { git = "https://github.com/astral-test/uv-public-pypackage", tag = "0.0.1" }
         "###
         );
     });
@@ -344,18 +485,14 @@ fn add_unnamed() -> Result<()> {
         name = "project"
         version = "0.1.0"
         source = "editable+."
-        sdist = { path = "." }
 
         [[distribution.dependencies]]
         name = "uv-public-pypackage"
-        version = "0.1.0"
-        source = "git+https://github.com/astral-test/uv-public-pypackage?rev=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979"
 
         [[distribution]]
         name = "uv-public-pypackage"
         version = "0.1.0"
-        source = "git+https://github.com/astral-test/uv-public-pypackage?rev=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979"
-        sdist = { url = "https://github.com/astral-test/uv-public-pypackage?rev=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979" }
+        source = "git+https://github.com/astral-test/uv-public-pypackage?tag=0.0.1#0dacfd662c64cb4ceb16e6cf65a157a8b715b979"
         "###
         );
     });
@@ -374,9 +511,9 @@ fn add_unnamed() -> Result<()> {
     Ok(())
 }
 
-/// Add a development dependency.
+/// Add and remove a development dependency.
 #[test]
-fn add_dev() -> Result<()> {
+fn add_remove_dev() -> Result<()> {
     let context = TestContext::new("3.12");
 
     let pyproject_toml = context.temp_dir.child("pyproject.toml");
@@ -396,10 +533,10 @@ fn add_dev() -> Result<()> {
     ----- stderr -----
     warning: `uv add` is experimental and may change without warning.
     Resolved 4 packages in [TIME]
-    Downloaded 4 packages in [TIME]
+    Prepared 4 packages in [TIME]
     Installed 4 packages in [TIME]
      + anyio==3.7.0
-     + idna==3.7
+     + idna==3.6
      + project==0.1.0 (from file://[TEMP_DIR]/)
      + sniffio==1.3.1
     "###);
@@ -445,33 +582,26 @@ fn add_dev() -> Result<()> {
 
         [[distribution.dependencies]]
         name = "idna"
-        version = "3.7"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution.dependencies]]
         name = "sniffio"
-        version = "1.3.1"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution]]
         name = "idna"
-        version = "3.7"
+        version = "3.6"
         source = "registry+https://pypi.org/simple"
-        sdist = { url = "https://files.pythonhosted.org/packages/21/ed/f86a79a07470cb07819390452f178b3bef1d375f2ec021ecfc709fc7cf07/idna-3.7.tar.gz", hash = "sha256:028ff3aadf0609c1fd278d8ea3089299412a7a8b9bd005dd08b9f8285bcb5cfc", size = 189575 }
-        wheels = [{ url = "https://files.pythonhosted.org/packages/e5/3e/741d8c82801c347547f8a2a06aa57dbb1992be9e948df2ea0eda2c8b79e8/idna-3.7-py3-none-any.whl", hash = "sha256:82fee1fc78add43492d3a1898bfa6d8a904cc97d8427f683ed8e798d07761aa0", size = 66836 }]
+        sdist = { url = "https://files.pythonhosted.org/packages/bf/3f/ea4b9117521a1e9c50344b909be7886dd00a519552724809bb1f486986c2/idna-3.6.tar.gz", hash = "sha256:9ecdbbd083b06798ae1e86adcbfe8ab1479cf864e4ee30fe4e46a003d12491ca", size = 175426 }
+        wheels = [{ url = "https://files.pythonhosted.org/packages/c2/e7/a82b05cf63a603df6e68d59ae6a68bf5064484a0718ea5033660af4b54a9/idna-3.6-py3-none-any.whl", hash = "sha256:c05567e9c24a6b9faaa835c4821bad0590fbb9d5779e7caa6e1cc4978e7eb24f", size = 61567 }]
 
         [[distribution]]
         name = "project"
         version = "0.1.0"
         source = "editable+."
-        sdist = { path = "." }
 
         [distribution.dev-dependencies]
 
         [[distribution.dev-dependencies.dev]]
         name = "anyio"
-        version = "3.7.0"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution]]
         name = "sniffio"
@@ -494,6 +624,396 @@ fn add_dev() -> Result<()> {
     Audited 4 packages in [TIME]
     "###);
 
+    // This should fail without --dev.
+    uv_snapshot!(context.filters(), context.remove(&["anyio"]), @r###"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv remove` is experimental and may change without warning.
+    warning: `anyio` is a development dependency; try calling `uv remove --dev`
+    error: The dependency `anyio` could not be found in `dependencies`
+    "###);
+
+    // Remove the dependency.
+    uv_snapshot!(context.filters(), context.remove(&["anyio"]).arg("--dev"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv remove` is experimental and may change without warning.
+    Resolved 1 package in [TIME]
+    Prepared 1 package in [TIME]
+    Uninstalled 4 packages in [TIME]
+    Installed 1 package in [TIME]
+     - anyio==3.7.0
+     - idna==3.6
+     - project==0.1.0 (from file://[TEMP_DIR]/)
+     + project==0.1.0 (from file://[TEMP_DIR]/)
+     - sniffio==1.3.1
+    "###);
+
+    let pyproject_toml = fs_err::read_to_string(context.temp_dir.join("pyproject.toml"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            pyproject_toml, @r###"
+        [project]
+        name = "project"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = []
+
+        [tool.uv]
+        dev-dependencies = []
+        "###
+        );
+    });
+
+    let lock = fs_err::read_to_string(context.temp_dir.join("uv.lock"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            lock, @r###"
+        version = 1
+        requires-python = ">=3.12"
+
+        [[distribution]]
+        name = "project"
+        version = "0.1.0"
+        source = "editable+."
+        "###
+        );
+    });
+
+    // Install from the lockfile.
+    uv_snapshot!(context.filters(), context.sync(), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv sync` is experimental and may change without warning.
+    Audited 1 package in [TIME]
+    "###);
+
+    Ok(())
+}
+
+/// Add and remove a workspace dependency.
+#[test]
+fn add_remove_workspace() -> Result<()> {
+    let context = TestContext::new("3.12");
+
+    let workspace = context.temp_dir.child("pyproject.toml");
+    workspace.write_str(indoc! {r#"
+        [tool.uv.workspace]
+        members = ["child1", "child2"]
+    "#})?;
+
+    let pyproject_toml = context.temp_dir.child("child1/pyproject.toml");
+    pyproject_toml.write_str(indoc! {r#"
+        [project]
+        name = "child1"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = []
+    "#})?;
+
+    let pyproject_toml = context.temp_dir.child("child2/pyproject.toml");
+    pyproject_toml.write_str(indoc! {r#"
+        [project]
+        name = "child2"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = []
+    "#})?;
+
+    // Adding a workspace package with a mismatched source should error.
+    let mut add_cmd =
+        context.add(&["child2 @ git+https://github.com/astral-test/uv-public-pypackage"]);
+    add_cmd
+        .arg("--preview")
+        .arg("--package")
+        .arg("child1")
+        .current_dir(&context.temp_dir);
+
+    uv_snapshot!(context.filters(), add_cmd, @r###"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    error: Workspace dependency `child2` must refer to local directory, not a Git repository
+    "###);
+
+    // Workspace packages should be detected automatically.
+    let child1 = context.temp_dir.join("child1");
+    let mut add_cmd = context.add(&["child2"]);
+    add_cmd
+        .arg("--preview")
+        .arg("--package")
+        .arg("child1")
+        .current_dir(&context.temp_dir);
+
+    uv_snapshot!(context.filters(), add_cmd, @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Resolved 2 packages in [TIME]
+    Prepared 2 packages in [TIME]
+    Installed 2 packages in [TIME]
+     + child1==0.1.0 (from file://[TEMP_DIR]/child1)
+     + child2==0.1.0 (from file://[TEMP_DIR]/child2)
+    "###);
+
+    let pyproject_toml = fs_err::read_to_string(child1.join("pyproject.toml"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            pyproject_toml, @r###"
+        [project]
+        name = "child1"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = [
+            "child2",
+        ]
+
+        [tool.uv.sources]
+        child2 = { workspace = true }
+        "###
+        );
+    });
+
+    // `uv add` implies a full lock and sync, including development dependencies.
+    let lock = fs_err::read_to_string(context.temp_dir.join("uv.lock"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            lock, @r###"
+        version = 1
+        requires-python = ">=3.12"
+
+        [[distribution]]
+        name = "child1"
+        version = "0.1.0"
+        source = "editable+child1"
+
+        [[distribution.dependencies]]
+        name = "child2"
+
+        [[distribution]]
+        name = "child2"
+        version = "0.1.0"
+        source = "editable+child2"
+        "###
+        );
+    });
+
+    // Install from the lockfile.
+    uv_snapshot!(context.filters(), context.sync().current_dir(&child1), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv sync` is experimental and may change without warning.
+    Audited 2 packages in [TIME]
+    "###);
+
+    // Remove the dependency.
+    uv_snapshot!(context.filters(), context.remove(&["child2"]).current_dir(&child1), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv remove` is experimental and may change without warning.
+    Resolved 2 packages in [TIME]
+    Prepared 1 package in [TIME]
+    Uninstalled 2 packages in [TIME]
+    Installed 1 package in [TIME]
+     - child1==0.1.0 (from file://[TEMP_DIR]/child1)
+     + child1==0.1.0 (from file://[TEMP_DIR]/child1)
+     - child2==0.1.0 (from file://[TEMP_DIR]/child2)
+    "###);
+
+    let pyproject_toml = fs_err::read_to_string(child1.join("pyproject.toml"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            pyproject_toml, @r###"
+        [project]
+        name = "child1"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = []
+
+        [tool.uv.sources]
+        "###
+        );
+    });
+
+    let lock = fs_err::read_to_string(context.temp_dir.join("uv.lock"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            lock, @r###"
+        version = 1
+        requires-python = ">=3.12"
+
+        [[distribution]]
+        name = "child1"
+        version = "0.1.0"
+        source = "editable+child1"
+
+        [[distribution]]
+        name = "child2"
+        version = "0.1.0"
+        source = "editable+child2"
+        "###
+        );
+    });
+
+    // Install from the lockfile.
+    uv_snapshot!(context.filters(), context.sync().current_dir(&child1), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv sync` is experimental and may change without warning.
+    Audited 1 package in [TIME]
+    "###);
+
+    Ok(())
+}
+
+/// Add a workspace dependency as an editable.
+#[test]
+fn add_workspace_editable() -> Result<()> {
+    let context = TestContext::new("3.12");
+
+    let workspace = context.temp_dir.child("pyproject.toml");
+    workspace.write_str(indoc! {r#"
+        [tool.uv.workspace]
+        members = ["child1", "child2"]
+    "#})?;
+
+    let pyproject_toml = context.temp_dir.child("child1/pyproject.toml");
+    pyproject_toml.write_str(indoc! {r#"
+        [project]
+        name = "child1"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = []
+    "#})?;
+
+    let pyproject_toml = context.temp_dir.child("child2/pyproject.toml");
+    pyproject_toml.write_str(indoc! {r#"
+        [project]
+        name = "child2"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = []
+    "#})?;
+
+    let child1 = context.temp_dir.join("child1");
+    let mut add_cmd = context.add(&["child2"]);
+    add_cmd
+        .arg("--editable")
+        .arg("--preview")
+        .current_dir(&child1);
+
+    uv_snapshot!(context.filters(), add_cmd, @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    Resolved 2 packages in [TIME]
+    Prepared 2 packages in [TIME]
+    Installed 2 packages in [TIME]
+     + child1==0.1.0 (from file://[TEMP_DIR]/child1)
+     + child2==0.1.0 (from file://[TEMP_DIR]/child2)
+    "###);
+
+    let pyproject_toml = fs_err::read_to_string(child1.join("pyproject.toml"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            pyproject_toml, @r###"
+        [project]
+        name = "child1"
+        version = "0.1.0"
+        requires-python = ">=3.12"
+        dependencies = [
+            "child2",
+        ]
+
+        [tool.uv.sources]
+        child2 = { workspace = true, editable = true }
+        "###
+        );
+    });
+
+    // `uv add` implies a full lock and sync, including development dependencies.
+    let lock = fs_err::read_to_string(context.temp_dir.join("uv.lock"))?;
+
+    insta::with_settings!({
+        filters => context.filters(),
+    }, {
+        assert_snapshot!(
+            lock, @r###"
+        version = 1
+        requires-python = ">=3.12"
+
+        [[distribution]]
+        name = "child1"
+        version = "0.1.0"
+        source = "editable+child1"
+
+        [[distribution.dependencies]]
+        name = "child2"
+
+        [[distribution]]
+        name = "child2"
+        version = "0.1.0"
+        source = "editable+child2"
+        "###
+        );
+    });
+
+    // Install from the lockfile.
+    uv_snapshot!(context.filters(), context.sync().current_dir(&child1), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+
+    ----- stderr -----
+    warning: `uv sync` is experimental and may change without warning.
+    Audited 2 packages in [TIME]
+    "###);
+
     Ok(())
 }
 
@@ -531,7 +1051,7 @@ fn update_registry() -> Result<()> {
 
     ----- stderr -----
     warning: `uv sync` is experimental and may change without warning.
-    Downloaded 4 packages in [TIME]
+    Prepared 4 packages in [TIME]
     Installed 4 packages in [TIME]
      + anyio==3.7.0
      + idna==3.6
@@ -547,7 +1067,7 @@ fn update_registry() -> Result<()> {
     ----- stderr -----
     warning: `uv add` is experimental and may change without warning.
     Resolved 4 packages in [TIME]
-    Downloaded 2 packages in [TIME]
+    Prepared 2 packages in [TIME]
     Uninstalled 2 packages in [TIME]
     Installed 2 packages in [TIME]
      - anyio==3.7.0
@@ -593,13 +1113,9 @@ fn update_registry() -> Result<()> {
 
         [[distribution.dependencies]]
         name = "idna"
-        version = "3.6"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution.dependencies]]
         name = "sniffio"
-        version = "1.3.1"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution]]
         name = "idna"
@@ -612,12 +1128,9 @@ fn update_registry() -> Result<()> {
         name = "project"
         version = "0.1.0"
         source = "editable+."
-        sdist = { path = "." }
 
         [[distribution.dependencies]]
         name = "anyio"
-        version = "4.3.0"
-        source = "registry+https://pypi.org/simple"
 
         [[distribution]]
         name = "sniffio"
@@ -676,7 +1189,7 @@ fn add_no_clean() -> Result<()> {
 
     ----- stderr -----
     warning: `uv sync` is experimental and may change without warning.
-    Downloaded 4 packages in [TIME]
+    Prepared 4 packages in [TIME]
     Installed 4 packages in [TIME]
      + anyio==3.7.0
      + idna==3.6
@@ -701,7 +1214,7 @@ fn add_no_clean() -> Result<()> {
     ----- stderr -----
     warning: `uv add` is experimental and may change without warning.
     Resolved 2 packages in [TIME]
-    Downloaded 2 packages in [TIME]
+    Prepared 2 packages in [TIME]
     Uninstalled 1 package in [TIME]
     Installed 2 packages in [TIME]
      + iniconfig==2.0.0
@@ -748,12 +1261,9 @@ fn add_no_clean() -> Result<()> {
         name = "project"
         version = "0.1.0"
         source = "editable+."
-        sdist = { path = "." }
 
         [[distribution.dependencies]]
         name = "iniconfig"
-        version = "2.0.0"
-        source = "registry+https://pypi.org/simple"
         "###
         );
     });
@@ -817,7 +1327,7 @@ fn remove_registry() -> Result<()> {
 
     ----- stderr -----
     warning: `uv sync` is experimental and may change without warning.
-    Downloaded 4 packages in [TIME]
+    Prepared 4 packages in [TIME]
     Installed 4 packages in [TIME]
      + anyio==3.7.0
      + idna==3.6
@@ -833,7 +1343,7 @@ fn remove_registry() -> Result<()> {
     ----- stderr -----
     warning: `uv remove` is experimental and may change without warning.
     Resolved 1 package in [TIME]
-    Downloaded 1 package in [TIME]
+    Prepared 1 package in [TIME]
     Uninstalled 4 packages in [TIME]
     Installed 1 package in [TIME]
      - anyio==3.7.0
@@ -873,239 +1383,6 @@ fn remove_registry() -> Result<()> {
         name = "project"
         version = "0.1.0"
         source = "editable+."
-        sdist = { path = "." }
-        "###
-        );
-    });
-
-    // Install from the lockfile.
-    uv_snapshot!(context.filters(), context.sync(), @r###"
-    success: true
-    exit_code: 0
-    ----- stdout -----
-
-    ----- stderr -----
-    warning: `uv sync` is experimental and may change without warning.
-    Audited 1 package in [TIME]
-    "###);
-
-    Ok(())
-}
-
-/// Remove a development dependency.
-#[test]
-fn remove_dev() -> Result<()> {
-    let context = TestContext::new("3.12");
-
-    let pyproject_toml = context.temp_dir.child("pyproject.toml");
-    pyproject_toml.write_str(indoc! {r#"
-        [project]
-        name = "project"
-        version = "0.1.0"
-        requires-python = ">=3.12"
-        dependencies = []
-
-        [tool.uv]
-        dev-dependencies = ["anyio==3.7.0"]
-    "#})?;
-
-    uv_snapshot!(context.filters(), context.lock(), @r###"
-    success: true
-    exit_code: 0
-    ----- stdout -----
-
-    ----- stderr -----
-    warning: `uv lock` is experimental and may change without warning.
-    Resolved 4 packages in [TIME]
-    "###);
-
-    uv_snapshot!(context.filters(), context.sync(), @r###"
-    success: true
-    exit_code: 0
-    ----- stdout -----
-
-    ----- stderr -----
-    warning: `uv sync` is experimental and may change without warning.
-    Downloaded 4 packages in [TIME]
-    Installed 4 packages in [TIME]
-     + anyio==3.7.0
-     + idna==3.6
-     + project==0.1.0 (from file://[TEMP_DIR]/)
-     + sniffio==1.3.1
-    "###);
-
-    uv_snapshot!(context.filters(), context.remove(&["anyio"]), @r###"
-    success: false
-    exit_code: 2
-    ----- stdout -----
-
-    ----- stderr -----
-    warning: `uv remove` is experimental and may change without warning.
-    warning: `anyio` is a development dependency; try calling `uv add --dev`
-    error: The dependency `anyio` could not be found in `dependencies`
-    "###);
-
-    uv_snapshot!(context.filters(), context.remove(&["anyio"]).arg("--dev"), @r###"
-    success: true
-    exit_code: 0
-    ----- stdout -----
-
-    ----- stderr -----
-    warning: `uv remove` is experimental and may change without warning.
-    Resolved 1 package in [TIME]
-    Downloaded 1 package in [TIME]
-    Uninstalled 4 packages in [TIME]
-    Installed 1 package in [TIME]
-     - anyio==3.7.0
-     - idna==3.6
-     - project==0.1.0 (from file://[TEMP_DIR]/)
-     + project==0.1.0 (from file://[TEMP_DIR]/)
-     - sniffio==1.3.1
-    "###);
-
-    let pyproject_toml = fs_err::read_to_string(context.temp_dir.join("pyproject.toml"))?;
-
-    insta::with_settings!({
-        filters => context.filters(),
-    }, {
-        assert_snapshot!(
-            pyproject_toml, @r###"
-        [project]
-        name = "project"
-        version = "0.1.0"
-        requires-python = ">=3.12"
-        dependencies = []
-
-        [tool.uv]
-        dev-dependencies = []
-        "###
-        );
-    });
-
-    let lock = fs_err::read_to_string(context.temp_dir.join("uv.lock"))?;
-
-    insta::with_settings!({
-        filters => context.filters(),
-    }, {
-        assert_snapshot!(
-            lock, @r###"
-        version = 1
-        requires-python = ">=3.12"
-
-        [[distribution]]
-        name = "project"
-        version = "0.1.0"
-        source = "editable+."
-        sdist = { path = "." }
-        "###
-        );
-    });
-
-    // Install from the lockfile.
-    uv_snapshot!(context.filters(), context.sync(), @r###"
-    success: true
-    exit_code: 0
-    ----- stdout -----
-
-    ----- stderr -----
-    warning: `uv sync` is experimental and may change without warning.
-    Audited 1 package in [TIME]
-    "###);
-
-    Ok(())
-}
-
-/// Remove a PyPI requirement that occurs multiple times.
-#[test]
-fn remove_all_registry() -> Result<()> {
-    let context = TestContext::new("3.12");
-
-    let pyproject_toml = context.temp_dir.child("pyproject.toml");
-    pyproject_toml.write_str(indoc! {r#"
-        [project]
-        name = "project"
-        version = "0.1.0"
-        requires-python = ">=3.12"
-        dependencies = [
-          "anyio == 3.7.0 ; python_version >= '3.12'",
-          "anyio < 3.7.0 ; python_version < '3.12'",
-        ]
-    "#})?;
-
-    uv_snapshot!(context.filters(), context.lock(), @r###"
-    success: true
-    exit_code: 0
-    ----- stdout -----
-
-    ----- stderr -----
-    warning: `uv lock` is experimental and may change without warning.
-    Resolved 4 packages in [TIME]
-    "###);
-
-    uv_snapshot!(context.filters(), context.sync(), @r###"
-    success: true
-    exit_code: 0
-    ----- stdout -----
-
-    ----- stderr -----
-    warning: `uv sync` is experimental and may change without warning.
-    Downloaded 4 packages in [TIME]
-    Installed 4 packages in [TIME]
-     + anyio==3.7.0
-     + idna==3.6
-     + project==0.1.0 (from file://[TEMP_DIR]/)
-     + sniffio==1.3.1
-    "###);
-
-    uv_snapshot!(context.filters(), context.remove(&["anyio"]), @r###"
-    success: true
-    exit_code: 0
-    ----- stdout -----
-
-    ----- stderr -----
-    warning: `uv remove` is experimental and may change without warning.
-    Resolved 1 package in [TIME]
-    Downloaded 1 package in [TIME]
-    Uninstalled 4 packages in [TIME]
-    Installed 1 package in [TIME]
-     - anyio==3.7.0
-     - idna==3.6
-     - project==0.1.0 (from file://[TEMP_DIR]/)
-     + project==0.1.0 (from file://[TEMP_DIR]/)
-     - sniffio==1.3.1
-    "###);
-
-    let pyproject_toml = fs_err::read_to_string(context.temp_dir.join("pyproject.toml"))?;
-
-    insta::with_settings!({
-        filters => context.filters(),
-    }, {
-        assert_snapshot!(
-            pyproject_toml, @r###"
-        [project]
-        name = "project"
-        version = "0.1.0"
-        requires-python = ">=3.12"
-        dependencies = []
-        "###
-        );
-    });
-
-    let lock = fs_err::read_to_string(context.temp_dir.join("uv.lock"))?;
-
-    insta::with_settings!({
-        filters => context.filters(),
-    }, {
-        assert_snapshot!(
-            lock, @r###"
-        version = 1
-        requires-python = ">=3.12"
-
-        [[distribution]]
-        name = "project"
-        version = "0.1.0"
-        source = "editable+."
-        sdist = { path = "." }
         "###
         );
     });
